<?php

namespace Drupal\aws_cloud\Controller\Vpc;

use Drupal\aws_cloud\Service\AwsEc2ServiceInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Render\RendererInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Controller responsible for "update" urls.
 *
 * This class is mainly responsible for
 * updating the aws entities from urls.
 */
class ApiController extends ControllerBase implements ApiControllerInterface {

  /**
   * The Aws Ec2 Service.
   *
   * @var \Drupal\aws_cloud\Service\AwsEc2ServiceInterface
   */
  private $awsEc2Service;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  private $requestStack;

  /**
   * Renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  private $renderer;

  /**
   * ApiController constructor.
   *
   * @param \Drupal\aws_cloud\Service\AwsEc2ServiceInterface $aws_ec2_service
   *   Object for interfacing with AWS Api.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   Messanger Object.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Request stack object.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(
    AwsEc2ServiceInterface $aws_ec2_service,
    Messenger $messenger,
    RequestStack $request_stack,
    RendererInterface $renderer
  ) {
    $this->awsEc2Service = $aws_ec2_service;
    $this->messenger = $messenger;
    $this->requestStack = $request_stack;
    $this->renderer = $renderer;
  }

  /**
   * Dependency Injection.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('aws_cloud.ec2'),
      $container->get('messenger'),
      $container->get('request_stack'),
      $container->get('renderer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function updateVpcList($cloud_context) {

    $this->awsEc2Service->setCloudContext($cloud_context);
    $updated = $this->awsEc2Service->updateVpcs();

    if ($updated !== FALSE) {
      $this->messenger->addMessage($this->t('Updated VPCs.'));
    }
    else {
      $this->messenger->addError($this->t('Unable to update VPCs.'));
    }

    return $this->redirect('view.aws_cloud_vpc.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function updateSubnetList($cloud_context) {

    $this->awsEc2Service->setCloudContext($cloud_context);
    $updated = $this->awsEc2Service->updateSubnets();

    if ($updated !== FALSE) {
      $this->messenger->addMessage($this->t('Updated Subnets.'));
    }
    else {
      $this->messenger->addError($this->t('Unable to update Subnets.'));
    }

    return $this->redirect('view.aws_cloud_subnet.list', [
      'cloud_context' => $cloud_context,
    ]);
  }

}
