<?php

namespace Drupal\cloud\Controller;

use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\cloud\Traits\AccessCheckTrait;

/**
 * Access controller for the Cloud config entity.
 *
 * @see \Drupal\cloud\Entity\CloudConfig.
 */
class CloudConfigAccessControlHandler extends EntityAccessControlHandler {

  use AccessCheckTrait;

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    /** @var \Drupal\cloud\Entity\CloudConfigInterface $entity */
    switch ($operation) {
      case 'view':
        if (!$entity->isPublished()) {
          return $this->allowedIfCanAccessCloudConfigWithOwner(
            $entity,
            $account,
            'view own unpublished cloud config entities',
            'view unpublished cloud config entities'
          );
        }

        return $this->allowedIfCanAccessCloudConfigWithOwner(
          $entity,
          $account,
          'view own published cloud config entities',
          'view published cloud config entities'
        );

      case 'update':
        return $this->allowedIfCanAccessCloudConfigWithOwner(
          $entity,
          $account,
          'edit own cloud config entities',
          'edit cloud config entities'
        );

      case 'delete':
        return $this->allowedIfCanAccessCloudConfigWithOwner(
          $entity,
          $account,
          'delete own cloud config entities',
          'delete cloud config entities'
        );
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

}
